/*
   Final program
   This program simulates a ball bouncing off of numerous pegs
   and polygons.  

author:Dustin Rhodes
email:dcrhodes@ucsc.edu
 */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <GL/glut.h>         /* glut.h includes gl.h and glu.h*/
//#include <GLUT/glut.h>
#include "Objects.h"



vector<Object*> objects;// = new vector<Object*>();
vector<Object*> balls;// = new vector<Object*>();
vector<Object*> pegs;// = new vector<Object*>();
vector<Line*> lines;

int mode=0;
int rotation=0;
int color=1;
int running=1;
float sphereX=0;
float sphereY=0;
float sphereZ=0;
void perspective();

float grav=-0.00006;
//displays everything and updates the physics
void display(void)
{
        glLoadIdentity();
        int i;
        for(i=0;i<objects.size();i++){
                glLoadIdentity();
                objects[i]->draw();
        }
        if(running){
            int j;
            for(i=0;i<balls.size();i++){
                balls[i]->addForce(0,grav);
                for(j=0;j<lines.size();j++){
                    lines[j]->hit(balls[i]);
                }
                for(j=0;j<pegs.size();j++){
                    balls[i]->hit(pegs[j]);
                }
                for(j=0;j<balls.size();j++){
                    if(i!=j)balls[i]->hit(balls[j]);
                }
                if(balls[i]->y<-10){
                    balls[i]->y=10;
                }
                if(balls[i]->x>10)balls[i]->x=-10;
                if(balls[i]->x<-10)balls[i]->x=10;
                
            }
        }else{
            for(i=0;i<objects.size();i++){
               objects[i]->vX=0;
               objects[i]->vY=0;
            }
        }
        glFlush();
        glutSwapBuffers();

}

int clicked=-1;
void mouseClick(int button, int state, int x, int y){
        x=x-400;
        y=y-400;
        float fx=x/40.0;
        float fy=y/40.0;
        fy*=-1;
        int i;
        for(i=0;i<objects.size();i++){
                if(objects[i]->mouseHit(fx,fy)){
                        clicked=i;
//                        break;
                }
        }
        if(state==1){
                clicked=-1;
        }
}

void mouseMove(int x, int y){
        if(clicked!=-1){
            x=x-400;
            y=y-400;
            float fx=x/40.0;
            float fy=y/40.0;
            fy*=-1;
            objects[clicked]->move(fx,fy);
        }
}


//clears the screen and redraws it
void update(){
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        display();
        glutPostRedisplay();
}

//these three methods handle the creating of polygons from lines
Point* oldLine = new Point(0,0,0);
Point* start;
void newStart(float x, float y){
    oldLine = new Point(x,y,0);
    start = oldLine;
}

void endLine(){
   Line* toAdd = new Line(); 
   toAdd->start = oldLine;
   toAdd->end = start;
   oldLine=toAdd->end;
   objects.push_back(toAdd);
   lines.push_back(toAdd);
}

void addNewLine(float x, float y){
   Line* toAdd = new Line(); 
   toAdd->start = oldLine;
   toAdd->end = new Point(x,y,0);
   oldLine=toAdd->end;
   objects.push_back(toAdd);
   lines.push_back(toAdd);
}
//initialize the enterprise
void init()
{
      int i;
      int width=7;
      int height=7;
      float dW = 20.0/width;
      float dH = 18.0/height;
      struct timeval time;
      gettimeofday(&time,NULL);
      srand48(time.tv_usec);
      for(i=0;i<width*height;i++){
       Ball* toAdd = new Ball();
       toAdd->x=-10+(i%width)*dW;//+drand48()*1-0.5;//(i*4)%20-10;
       if((i/width)%2==1)toAdd->x+=dW/2;
       toAdd->y=-8+(i/width)*dH;//+drand48()*1-0.5;//((i*4)/20)*4-10;
       toAdd->y+=dH/2;
       toAdd->x+=dW/2;
       int type = drand48()*3;
       if(type==0)toAdd->moveable=0;
       if(type==1)toAdd->moveable=2;
       if(type==2)toAdd->moveable=3;
       toAdd->r=drand48()*.8+0.2;
       objects.push_back(toAdd);
       pegs.push_back(toAdd);
      }
      int numBalls = 10;
      for(i=0;i<numBalls;i++){
        Ball* toAdd = new Ball();
        toAdd->x=drand48()*20.0-10;
        toAdd->y=9+drand48()*1-0.5;
        toAdd->moveable=1;
        toAdd->r=0.3;
        objects.push_back(toAdd);
        balls.push_back(toAdd);
      }
    
      newStart(-9.5,-9.5);
      addNewLine(-9.5,-8);
      addNewLine(-2,-9.5);
      endLine();

      newStart(9.5,-9.5);
      addNewLine(9.5,-8);
      addNewLine(2,-9.5);
      endLine();

        perspective();
        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LEQUAL);
        glClearDepth(1.0);

        float diffuse0[]={0.5,0,0.0,0.0};
        float ambient0[]={0.2,0.2,0.2,0.0};
        float specular0[]={1.0,0.0,0.0,1.0};
        float light0_pos[]={0.0,5.0,0.0,1.0};
        float light0_dir[]={0.0,-1.0,0.0,1.0};

        glEnable(GL_CULL_FACE);

        float material[]={0.5,0.5,0.5,1};
        glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,material);
        glShadeModel(GL_SMOOTH);
        glEnable(GL_LIGHTING);
        glEnable(GL_LIGHT0);

        glLightfv(GL_LIGHT0, GL_POSITION, light0_pos);
        glLightfv(GL_LIGHT0, GL_AMBIENT, ambient0);
        glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse0);
        glLightfv(GL_LIGHT0, GL_SPECULAR, specular0);
        glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, light0_dir);

        float diffuse1[]={1,1,1,0.0};
        float ambient1[]={0,0,0,0};
        float specular1[]={0,1.0,0.0,0};
        float light1_pos[]={0,0,0,1};
        float light1_dir[]={0,-1,0.0,1};
        glEnable(GL_LIGHT1);

        glLightfv(GL_LIGHT1, GL_POSITION, light1_pos);
        glLightfv(GL_LIGHT1, GL_AMBIENT, ambient1);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuse1);
        glLightfv(GL_LIGHT1, GL_SPECULAR, specular1);
        glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, light1_dir);
        glEnable(GL_COLOR_MATERIAL);

}

//update the perspective with appropriate
//position and angles
void perspective(){
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        glOrtho(-10,10,-10,10,-10,10);
        glMatrixMode(GL_MODELVIEW);
}


void keyPressed(unsigned char key, int x, int y){

        if(key=='+' || key=='='){
                grav-=0.00001;
        }else if(key=='-'){
                grav+=0.00001;
        }else if(key=='b'){
            ;
                //ball->bounce=!ball->bounce;
        }else if(key==' '){
                running = !running;
        }else if(key=='a'){
            Ball* toAdd = new Ball();
            toAdd->x=drand48()*20.0-10;
            toAdd->y=9+drand48()*1-0.5;
            toAdd->moveable=1;
            toAdd->r=0.3;
            objects.push_back(toAdd);
            balls.push_back(toAdd);
        }
}

//handle changing the state out of rotation mode
void keyUp(unsigned char key, int x, int y){
}
int main(int argc, char** argv)
{
        glutInit(&argc, argv);
        glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
        glutInitWindowPosition(0,0);
        glutInitWindowSize(800,800);
        glutCreateWindow("prog2"); 
        glMatrixMode(GL_MODELVIEW);

        glutKeyboardFunc(keyPressed);
        glutKeyboardUpFunc(keyUp);
        glutDisplayFunc(update);
        glutMouseFunc(mouseClick);
        glutMotionFunc(mouseMove);
        glutPassiveMotionFunc(mouseMove);
        init();
        glutMainLoop();
//        glEnable(GL_LIGHTING);
        glEnable(GL_NORMALIZE);

}
